@@ -1,4 +1,4 @@
-//! Would you like to SAVE the change?
+#![doc = include_str!("../README.md")]
 #![warn(
     missing_docs,
     clippy::missing_docs_in_private_items,
@@ -12,7 +12,7 @@ use {
     ::{
         clap::Parser,
         digest::Digest,
-        eyre::{bail, Result as Fallible},
+        eyre::{bail, Result, WrapErr},
         git2::{ErrorCode, Repository, RepositoryInitOptions, Signature, Time},
         std::{env, fs, process::Command},
         tracing::{debug, info, trace, warn},
@@ -30,37 +30,47 @@ mod git;
 pub struct Args {
     /// The author email to use for the commit.
     ///
-    /// [default: email from git, or else from last commit, or else "save"]
+    /// [default: email from git, or else from parent commit, or else "save"]
     #[clap(long)]
-    email: Option<String>,
+    pub email: Option<String>,
+
+    /// The target commit hash or prefix, in hex.
+    ///
+    /// [default: the commit's tree hash]
+    #[clap(long = "hash", short = 'x')]
+    pub hash_hex: Option<String>,
 
     /// Use a manual commit message instead of the default generated message.
     #[clap(long, short)]
-    message: Option<String>,
+    pub message: Option<String>,
 
     /// The author name to use for the commit.
     ///
-    /// [default: name from git, or else from last commit, or else "save"]
+    /// [default: name from git, or else from parent commit, or else "save"]
     #[clap(long)]
-    name: Option<String>,
+    pub name: Option<String>,
 
     /// The time is NOW.
     ///
     /// [default: the time is ACTUALLY now]
     #[clap(long = "now")]
-    now_seconds: Option<i64>,
+    pub now_seconds: Option<i64>,
+
+    /// Decrease log verbosity. May be used multiple times.
+    #[clap(short, long, parse(from_occurrences))]
+    pub quiet: i32,
 
     /// Seconds of timestamp allocated for each commit to search.
     #[clap(long="step", default_value_t = 64 * 2)]
-    step_seconds: u32,
+    pub step_seconds: u32,
 
     /// Increase log verbosity. May be used multiple times.
     #[clap(short, long, parse(from_occurrences))]
-    verbose: u8,
+    pub verbose: i32,
 
     /// Proceed in spite of any warnings.
     #[clap(long, short)]
-    yes: bool,
+    pub yes: bool,
 }
 
 /// CLI entry point.
@@ -72,7 +82,7 @@ pub struct Args {
 /// # Errors
 ///
 /// For other fatal errors.
-pub fn main(args: Args) -> Fallible<()> {
+pub fn main(args: Args) -> Result<()> {
     trace!("{:#?}", args);
 
     let repo = match Repository::open_from_env() {
@@ -247,9 +257,15 @@ pub fn main(args: Args) -> Fallible<()> {
     let min_timestamp = seconds;
     let max_timestamp = seconds + step_seconds - 1;
 
-    let target_hash = tree.id().as_bytes().to_vec();
+    let mut target_hash = args
+        .hash_hex
+        .map(|s| hex::decode(s).wrap_err("target hash must be hex").unwrap())
+        .unwrap_or_default();
+    target_hash.append(&mut tree.id().as_bytes().to_vec());
 
     let trace_brute = debug_span!("Finding the most tree-like commit hash within step...");
+
+    // TODO: Do this ourselves, and make it parallel.
     let (_score, author_timestamp, commit_timestamp) =
         ((min_timestamp..=max_timestamp).map(|author_timestamp| {
             (author_timestamp..=max_timestamp)
@@ -347,10 +363,21 @@ pub fn init() -> Args {
 
     let args = Args::parse();
 
-    let log_level = match args.verbose {
-        0 => env::var("RUST_LOG").unwrap_or_else(|_| "info".into()),
-        1 => "debug".into(),
-        2.. => "trace".into(),
+    let default_verbosity = 3;
+
+    let log_env = env::var("RUST_LOG").unwrap_or_default();
+
+    let log_level = if args.verbose == 0 && args.quiet == 0 && !log_env.is_empty() {
+        log_env
+    } else {
+        match default_verbosity + args.verbose - args.quiet {
+            i32::MIN..=0 => "off".into(),
+            1 => "error".into(),
+            2 => "warn".into(),
+            3 => "info".into(),
+            4 => "debug".into(),
+            5..=i32::MAX => "trace".into(),
+        }
     };
 
     tracing_subscriber::util::SubscriberInitExt::init(tracing_subscriber::Layer::with_subscriber(
